@@ -18,6 +18,7 @@ from . import exceptions
 
 _scraper = FacebookScraper()
 
+
 def set_cookies(cookies):
     if isinstance(cookies, str):
         cookies = parse_cookie_file(cookies)
@@ -31,19 +32,28 @@ def set_cookies(cookies):
         if not _scraper.is_logged_in():
             raise exceptions.InvalidCookies(f"Cookies are not valid")
 
+
+def unset_cookies():
+    # Explicitly unset cookies to return to unauthenticated requests
+    _scraper.session.cookies = cookiejar_from_dict({})
+
+
 def set_proxy(proxy):
     _scraper.set_proxy(proxy)
 
+
 def set_user_agent(user_agent):
     _scraper.set_user_agent(user_agent)
 
+
 def set_noscript(noscript):
     _scraper.set_noscript(noscript)
 
+
 def get_profile(
     account: str,
     **kwargs,
- ) -> Profile:
+) -> Profile:
     """Get a Facebook user's profile information
     Args:
         account(str): The account of the profile.
@@ -55,10 +65,8 @@ def get_profile(
     set_cookies(cookies)
     return _scraper.get_profile(account, **kwargs)
 
-def get_page_info(
-    account: str,
-    **kwargs
-) -> Profile:
+
+def get_page_info(account: str, **kwargs) -> Profile:
     """Get a page's information
     Args:
         account(str): The account of the profile.
@@ -70,10 +78,8 @@ def get_page_info(
     set_cookies(cookies)
     return _scraper.get_page_info(account, **kwargs)
 
-def get_group_info(
-    group: Union[str, int],
-    **kwargs
-) -> Profile:
+
+def get_group_info(group: Union[str, int], **kwargs) -> Profile:
     """Get a group's profile information
     Args:
         group(str or int): The group name or ID
@@ -85,6 +91,7 @@ def get_group_info(
     set_cookies(cookies)
     return _scraper.get_group_info(group, **kwargs)
 
+
 def get_posts(
     account: Optional[str] = None,
     group: Union[str, int, None] = None,
@@ -283,7 +290,7 @@ def write_posts_to_csv(
     if encoding is None:
         encoding = locale.getpreferredencoding()
 
-    with open(filename, 'w', encoding=encoding) as output_file:
+    with open(filename, 'w', newline='', encoding=encoding) as output_file:
         dict_writer = csv.DictWriter(output_file, keys)
         dict_writer.writeheader()
         dict_writer.writerows(list_of_posts)
