@@ -27,20 +27,30 @@ logger = logging.getLogger(__name__)
 PartialPost = Optional[Dict[str, Any]]
 
 
-def extract_post(raw_post: RawPost, options: Options, request_fn: RequestFunction, full_post_html = None) -> Post:
+def extract_post(
+    raw_post: RawPost, options: Options, request_fn: RequestFunction, full_post_html=None
+) -> Post:
     return PostExtractor(raw_post, options, request_fn, full_post_html).extract_post()
 
-def extract_group_post(raw_post: RawPost, options: Options, request_fn: RequestFunction, full_post_html = None) -> Post:
+
+def extract_group_post(
+    raw_post: RawPost, options: Options, request_fn: RequestFunction, full_post_html=None
+) -> Post:
     return GroupPostExtractor(raw_post, options, request_fn, full_post_html).extract_post()
 
-def extract_photo_post(raw_post: RawPost, options: Options, request_fn: RequestFunction, full_post_html) -> Post:
+
+def extract_photo_post(
+    raw_post: RawPost, options: Options, request_fn: RequestFunction, full_post_html
+) -> Post:
     return PhotoPostExtractor(raw_post, options, request_fn, full_post_html).extract_post()
 
 
 class PostExtractor:
     """Class for Extracting fields from a FacebookPost"""
 
-    likes_regex = re.compile(r'([\d,.KM]+)\s+(Like|left reaction)', re.IGNORECASE)
+    likes_regex = re.compile(
+        r'([\d,.KM]+)\s+(Like|left reaction|others reacted|others left reactions)', re.IGNORECASE
+    )
     comments_regex = re.compile(r'([\d,.KM]+)\s+comment', re.IGNORECASE)
     shares_regex = re.compile(r'([\d,.KM]+)\s+Share', re.IGNORECASE)
     live_regex = re.compile(r'.+(is live).+')
@@ -52,7 +62,7 @@ class PostExtractor:
         r'<a href=\"([^\"]+?)\" target=\"_blank\" class=\"sec\">',
         re.IGNORECASE,
     )
-    image_regex_lq = re.compile(r"background-image: url\('(.+)'\)")
+    image_regex_lq = re.compile(r"url\('(.+)'\)")
     video_thumbnail_regex = re.compile(r"background: url\('(.+)'\)")
     post_url_regex = re.compile(r'/story.php\?story_fbid=')
     video_post_url_regex = re.compile(r'/.+/videos/.+/(.+)/.+')
@@ -64,10 +74,10 @@ class PostExtractor:
     )
     bad_json_key_regex = re.compile(r'(?P<prefix>[{,])(?P<key>\w+):')
 
-    more_url_regex = re.compile(r'(?<=…\s)<a href="([^"]+)')
+    more_url_regex = re.compile(r'(?<=…\s)<a')
     post_story_regex = re.compile(r'href="(\/story[^"]+)" aria')
 
-    def __init__(self, element, options, request_fn, full_post_html = None):
+    def __init__(self, element, options, request_fn, full_post_html=None):
         self.element = element
         self.options = options
         self.request = request_fn
@@ -147,7 +157,7 @@ class PostExtractor:
             self.extract_factcheck,
             self.extract_share_information,
             self.extract_availability,
-            self.extract_listing
+            self.extract_listing,
         ]
 
         post = self.make_new_post()
@@ -254,16 +264,10 @@ class PostExtractor:
             }
         elif element.find(".story_body_container>div", first=True):
             text = element.find(".story_body_container>div", first=True).text
-            return {
-                'text': text,
-                'post_text': text
-            }
+            return {'text': text, 'post_text': text}
         elif len(nodes) == 1:
             text = nodes[0].text
-            return {
-                'text': text,
-                'post_text': text
-            }
+            return {'text': text, 'post_text': text}
 
         return None
 
@@ -275,7 +279,9 @@ class PostExtractor:
         for page in page_insights.values():
             try:
                 timestamp = page['post_context']['publish_time']
-                logger.debug(f"Got exact timestamp from publish_time: {datetime.fromtimestamp(timestamp)}")
+                logger.debug(
+                    f"Got exact timestamp from publish_time: {datetime.fromtimestamp(timestamp)}"
+                )
                 return {
                     'time': datetime.fromtimestamp(timestamp),
                 }
@@ -297,7 +303,17 @@ class PostExtractor:
         if date:
             return {'time': date}
 
-        return None
+        try:
+            date_element = self.full_post_html.find("abbr[data-store*='time']", first=True)
+            time = json.loads(date_element.attrs["data-store"])["time"]
+            logger.debug(
+                f"Got exact timestamp from abbr[data-store]: {datetime.fromtimestamp(time)}"
+            )
+            return {
+                'time': datetime.fromtimestamp(time),
+            }
+        except:
+            return None
 
     def extract_user_id(self) -> PartialPost:
         return {'user_id': self.data_ft['content_owner_id_new']}
@@ -305,7 +321,7 @@ class PostExtractor:
     def extract_image_lq(self) -> PartialPost:
         elems = self.element.find('div.story_body_container>div .img:not(.profpic)')
         if not elems:
-            elems = self.element.find('#root .img:not(.profpic)')
+            elems = self.element.find('.img:not(.profpic), img:not(.profpic)')
         images = []
         descriptions = []
         for elem in elems:
@@ -321,7 +337,11 @@ class PostExtractor:
                 descriptions.append(elem.attrs.get("aria-label") or elem.attrs.get("alt"))
 
         image = images[0] if images else None
-        result = {"image_lowquality": image, "images_lowquality": images, "images_lowquality_description": descriptions}
+        result = {
+            "image_lowquality": image,
+            "images_lowquality": images,
+            "images_lowquality_description": descriptions,
+        }
         # Link to high resolution external image embedded in low quality image url
         if image and "safe_image.php" in image and not self.post.get("image"):
             url = parse_qs(urlparse(image).query).get("url")
@@ -381,6 +401,8 @@ class PostExtractor:
             )
             or self.live_data.get("like_count")
             or self.live_data.get("reactioncount")
+            or (self.element.find(".likes", first=True) and utils.parse_int(self.element.find(".likes", first=True).text))
+            or (self.element.find(".like_def", first=True) and utils.parse_int(self.element.find(".like_def", first=True).text))
             or 0,
         }
 
@@ -390,6 +412,7 @@ class PostExtractor:
                 self.element, 'footer', self.comments_regex, utils.convert_numeric_abbr
             )
             or self.live_data.get("comment_count")
+            or (self.element.find(".cmt_def", first=True) and utils.parse_int(self.element.find(".cmt_def", first=True).text))
             or 0,
         }
 
@@ -414,7 +437,11 @@ class PostExtractor:
                 logger.debug(f"Fetching {url}")
                 try:
                     redirect_response = self.request(url)
-                    url = redirect_response.html.find("a", first=True).attrs.get("href").replace("&amp;", "&")
+                    url = (
+                        redirect_response.html.find("a", first=True)
+                        .attrs.get("href")
+                        .replace("&amp;", "&")
+                    )
                 except Exception as e:
                     logger.error(e)
             return url
@@ -434,7 +461,7 @@ class PostExtractor:
         )
 
         total_photos_in_gallery = len(photo_links)
-        if len(photo_links) in [4,5] and photo_links[-1].text:
+        if len(photo_links) in [4, 5] and photo_links[-1].text:
             total_photos_in_gallery = len(photo_links) + int(photo_links[-1].text.strip("+")) - 1
             logger.debug(f"{total_photos_in_gallery} total photos in gallery")
 
@@ -469,7 +496,7 @@ class PostExtractor:
                     "video": videos[0] if videos else None,
                     "video_id": video_ids[0] if video_ids else None,
                     "video_ids": video_ids,
-                    "videos": videos
+                    "videos": videos,
                 }
             url = utils.urljoin(FB_MOBILE_BASE_URL, url)
             logger.debug(f"Fetching {url}")
@@ -490,7 +517,9 @@ class PostExtractor:
             if response.html.find("a", containing="Photos from", first=True):
                 # Right arrow link
                 direction = '{"tn":"+="}'
-            url = response.html.find(f"a.touchable[data-gt='{direction}']", first=True).attrs["href"]
+            url = response.html.find(f"a.touchable[data-gt='{direction}']", first=True).attrs[
+                "href"
+            ]
             if not url.startswith("http"):
                 url = utils.urljoin(FB_MOBILE_BASE_URL, url)
             logger.debug(f"Fetching {url}")
@@ -508,7 +537,13 @@ class PostExtractor:
                     break
         image = images[0] if images else None
         image_id = image_ids[0] if image_ids else None
-        return {"image": image, "images": images, "images_description": descriptions, "image_id": image_id, "image_ids": image_ids}
+        return {
+            "image": image,
+            "images": images,
+            "images_description": descriptions,
+            "image_id": image_id,
+            "image_ids": image_ids,
+        }
 
     def extract_reactions(self) -> PartialPost:
         """Fetch share and reactions information with a existing post obtained by `get_posts`.
@@ -530,7 +565,7 @@ class PostExtractor:
         reaction_lookup = self.get_jsmod("UFIReactionTypes")
         if reaction_lookup:
             reaction_lookup = reaction_lookup.get("reactions")
-            for k,v in self.live_data.get("reactioncountmap").items():
+            for k, v in self.live_data.get("reactioncountmap").items():
                 if v["default"]:
                     name = reaction_lookup[k]["display_name"].lower()
                     reactions[name] = v["default"]
@@ -558,7 +593,11 @@ class PostExtractor:
             if type(reactors_opt) in [int, float] and reactors_opt < limit:
                 limit = reactors_opt
             logger.debug(f"Fetching {limit} reactors")
-            elems = list(response.html.find("div#reaction_profile_browser>div,div#reaction_profile_browser1>div"))
+            elems = list(
+                response.html.find(
+                    "div#reaction_profile_browser>div,div#reaction_profile_browser1>div"
+                )
+            )
             more = response.html.find("div#reaction_profile_pager a", first=True)
             if more and limit > 50:
                 url = utils.urljoin(FB_MOBILE_BASE_URL, more.attrs.get("href"))
@@ -570,19 +609,30 @@ class PostExtractor:
 
                 for action in data['payload']['actions']:
                     if action['cmd'] == 'append':
-                        html = utils.make_html_element(f"<div id='reaction_profile_browser'>{action['html']}</div>", url=FB_MOBILE_BASE_URL)
-                        more_elems = html.find('div#reaction_profile_browser>div,div#reaction_profile_browser1>div')
+                        html = utils.make_html_element(
+                            f"<div id='reaction_profile_browser'>{action['html']}</div>",
+                            url=FB_MOBILE_BASE_URL,
+                        )
+                        more_elems = html.find(
+                            'div#reaction_profile_browser>div,div#reaction_profile_browser1>div'
+                        )
                         elems.extend(more_elems)
             logger.debug(f"Found {len(elems)} reactors")
             for elem in elems:
-                emoji_class = elem.find(f"div>i.{spriteMapCssClass}", first=True).attrs.get("class")[-1]
+                emoji_class = elem.find(f"div>i.{spriteMapCssClass}", first=True).attrs.get(
+                    "class"
+                )[-1]
                 if not emoji_class_lookup.get(emoji_class):
                     logger.error(f"Don't know {emoji_class}")
-                reactors.append({
-                    "name": elem.find("strong", first=True).text,
-                    "link": utils.urljoin(FB_BASE_URL, elem.find("a", first=True).attrs.get("href")),
-                    "type": emoji_class_lookup.get(emoji_class)
-                })
+                reactors.append(
+                    {
+                        "name": elem.find("strong", first=True).text,
+                        "link": utils.urljoin(
+                            FB_BASE_URL, elem.find("a", first=True).attrs.get("href")
+                        ),
+                        "type": emoji_class_lookup.get(emoji_class),
+                    }
+                )
 
         if reactions:
             return {
@@ -612,7 +662,7 @@ class PostExtractor:
                         'w3_fb_url': data['url'],
                         'fetched_time': datetime.now(),
                     }
-        return None
+        return {'fetched_time': datetime.now()}
 
     def extract_video(self):
         video_data_element = self.element.find('[data-sigil="inlineVideo"]', first=True)
@@ -622,13 +672,15 @@ class PostExtractor:
             video_id = query["photo"][0]
             logger.debug(f"Fetching {video_id}")
             response = self.request(video_id)
-            video_post = PostExtractor(response.html, self.options, self.request, full_post_html=response.html)
+            video_post = PostExtractor(
+                response.html, self.options, self.request, full_post_html=response.html
+            )
             video_post.post = {"post_id": video_id}
             meta = video_post.extract_video_meta() or {}
             return {
                 "video_id": video_id,
                 "video": video_post.extract_video().get("video"),
-                **meta
+                **meta,
             }
 
         if video_data_element is None:
@@ -761,25 +813,35 @@ class PostExtractor:
     def parse_comment(self, comment):
         comment_id = comment.attrs.get("id")
 
-        profile_picture = comment.find(".profpic.img", first=True)
-        name = profile_picture.attrs.get("alt") or profile_picture.attrs.get("aria-label")
-        name = name.split(",")[0]
-        commenter_id = re.search(r'feed_story_ring(\d+)', comment.html)
-        if commenter_id:
-            commenter_id = commenter_id.group(1)
+        try:
+            profile_picture = comment.find(".profpic.img", first=True)
+            name = profile_picture.attrs.get("alt") or profile_picture.attrs.get("aria-label")
+            name = name.split(",")[0]
+            commenter_id = re.search(r'feed_story_ring(\d+)', comment.html)
+            if commenter_id:
+                commenter_id = commenter_id.group(1)
 
-        url = profile_picture.element.getparent().attrib.get("href")
-        if url:
-            url = utils.urljoin(FB_BASE_URL, url)
-
-        first_link = comment.find("div:not([data-sigil])>a[href]:not([data-click]):not([data-store]):not([data-sigil])", first=True)
-        comment_body_elem = comment.find('[data-sigil="comment-body"]', first=True)
+            url = profile_picture.element.getparent().attrib.get("href")
+            if url:
+                url = utils.urljoin(FB_BASE_URL, url)
+        except AttributeError:
+            name = comment.find("h3", first=True).text
+            commenter_id = None
+            url = None
+        first_link = comment.find(
+            "div:not([data-sigil])>a[href]:not([data-click]):not([data-store]):not([data-sigil])",
+            first=True,
+        )
+        comment_body_elem = comment.find('[data-sigil="comment-body"],div._14ye,div.bl', first=True)
+        if comment_body_elem:
+            text = comment_body_elem.text
+        else:
+            text = comment.text
         commenter_meta = None
         if first_link:
             if "\n" in first_link.text:
                 commenter_meta = first_link.text.split("\n")[0]
 
-        text = comment_body_elem.text
         # Try to extract from the abbr element
         date_element = comment.find('abbr', first=True)
         if date_element:
@@ -808,17 +870,19 @@ class PostExtractor:
             "commenter_meta": commenter_meta,
             "comment_text": text,
             "comment_time": date,
-            "comment_image": image_url
+            "comment_image": image_url,
         }
 
     def extract_comments_full(self):
         """Fetch comments for an existing post obtained by `get_posts`.
         Note that this method may raise multiple http requests per post to get all comments"""
-        elem = self.full_post_html.find('div[data-sigil="m-mentions-expand"]', first=True)
-        if not elem:
+        comments_area_selector = 'div.ufi>div:last-child'
+        elem = self.full_post_html.find(comments_area_selector, first=True)
+        comments_selector = 'div[data-sigil="comment"],div._55wr'
+        comments = list(elem.find(comments_selector))
+        if not comments:
             logger.warning("No comments found on page")
             return
-        comments = list(elem.find('div[data-sigil="comment"]'))
 
         more_selector = f"a[href*='{self.post.get('post_id')}']"
         direction = "View more comments"
@@ -828,7 +892,7 @@ class PostExtractor:
             more = elem.find(more_selector, containing=direction, first=True)
 
         # Comment limiting and progress
-        limit = 5000 # Default
+        limit = 5000  # Default
         if more and more.attrs.get("data-ajaxify-href"):
             parsed = parse_qs(urlparse(more.attrs.get("data-ajaxify-href")).query)
             count = int(parsed.get("count")[0])
@@ -858,11 +922,11 @@ class PostExtractor:
                 logger.error(e)
                 break
             visited_urls.append(url)
-            elem = response.html.find('div[data-sigil="m-mentions-expand"]', first=True)
-            if not elem:
+            elem = response.html.find(comments_area_selector, first=True)
+            more_comments = elem.find(comments_selector)
+            if not more_comments:
                 logger.warning("No comments found on page")
                 break
-            more_comments = elem.find('div[data-sigil="comment"]')
             comments.extend(more_comments)
             more = elem.find(more_selector, containing=direction, first=True)
 
@@ -872,8 +936,14 @@ class PostExtractor:
             pbar.close()
             pbar = tqdm(total=len(comments))
         for comment in comments:
-            result = self.parse_comment(comment)
-            replies = comment.find("div.async_elem[data-sigil='replies-see-more'] a[href]", first=True)
+            try:
+                result = self.parse_comment(comment)
+            except Exception as e:
+                logger.error(f"Unable to parse comment {comment}: {e}")
+                continue
+            replies = comment.find(
+                "div.async_elem[data-sigil='replies-see-more'] a[href],div[id*='comment_replies_more'] a[href]", first=True
+            )
             if self.options.get("progress"):
                 pbar.update(1)
             if replies:
@@ -885,8 +955,15 @@ class PostExtractor:
                 except Exception as e:
                     logger.error(e)
                     continue
-                replies = response.html.find('div[data-sigil="comment"]')
-                result["replies"] = [self.parse_comment(reply) for reply in replies[1:]] # Skip first element, as it will be this comment itself
+                # Skip first element, as it will be this comment itself
+                replies = response.html.find('div[data-sigil="comment"],#root div[id]')[1:]
+                try:
+                    result["replies"] = [
+                        self.parse_comment(reply) for reply in replies
+                    ]
+                except Exception as e:
+                    logger.error(f"Unable to parse comment {result['comment_id']} replies {replies}: {e}")
+                    continue
             results.append(result)
         return {"comments_full": results}
 
@@ -903,7 +980,7 @@ class PostExtractor:
             return {
                 "listing_title": divs[0].find("span")[-1].text,
                 "listing_price": divs[1].text,
-                "listing_location": divs[2].text
+                "listing_location": divs[2].text,
             }
 
     @property
@@ -946,7 +1023,7 @@ class PostExtractor:
             logger.error(e)
         return self._live_data
 
-    def get_jsmod(self, name, element = None):
+    def get_jsmod(self, name, element=None):
         if not element:
             if self.full_post_html:
                 element = self.full_post_html
@@ -962,17 +1039,19 @@ class PostExtractor:
 
 class GroupPostExtractor(PostExtractor):
     """Class for extracting posts from Facebook Groups rather than Pages"""
+
     post_url_regex = re.compile(r'https://m.facebook.com/groups/[^/]+/permalink/')
     post_story_regex = re.compile(r'href="(https://m.facebook.com/groups/[^/]+/permalink/\d+/)')
 
+
 class PhotoPostExtractor(PostExtractor):
     def extract_text(self) -> PartialPost:
         text = self.element.find("div.msg", first=True).text
         return {"text": text, "post_text": text}
 
     def extract_photo_link(self) -> PartialPost:
-        image = self.extract_photo_link_HQ(self.element.html)
-        return {"image": image, "images": [image]}
+        image = self.extract_photo_link_HQ(self.full_post_html.html)
+        return {"image": image, "images": [image], "images_description": self.extract_image_lq()["images_lowquality_description"]}
 
     def extract_user_id(self) -> PartialPost:
         match = re.search(r'entity_id:(\d+),', self.element.html)
@@ -980,7 +1059,16 @@ class PhotoPostExtractor(PostExtractor):
             return {"user_id": match.group(1)}
 
     def extract_post_url(self) -> PartialPost:
-        return {"post_url": utils.urljoin(FB_MOBILE_BASE_URL, str(self.live_data["ft_ent_identifier"]))}
+        return {
+            "post_url": utils.urljoin(
+                FB_MOBILE_BASE_URL, self.extract_post_id()["post_id"]
+            )
+        }
 
     def extract_post_id(self) -> PartialPost:
-        return {"post_id": str(self.live_data["ft_ent_identifier"])}
+        try:
+            return {"post_id": str(self.live_data["ft_ent_identifier"])}
+        except KeyError:
+            match = re.search(r'ft_ent_identifier=(\d+)', self.full_post_html.html)
+            if match:
+                return {"post_id": match.groups()[0]}
