@@ -12,7 +12,7 @@ from requests_html import HTMLSession
 
 from . import utils
 from .constants import DEFAULT_PAGE_LIMIT, FB_BASE_URL, FB_MOBILE_BASE_URL, FB_W3_BASE_URL
-from .extractors import extract_group_post, extract_post, extract_photo_post
+from .extractors import extract_group_post, extract_post, extract_photo_post, PostExtractor
 from .fb_types import Post, Profile
 from .page_iterators import iter_group_pages, iter_pages, iter_photos
 from . import exceptions
@@ -28,7 +28,7 @@ class FacebookScraper:
     default_headers = {
         'Accept-Language': 'en-US,en;q=0.5',
         "Sec-Fetch-User": "?1",
-        "User-Agent": "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.212 Mobile Safari/537.36"
+        "User-Agent": "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.212 Mobile Safari/537.36",
     }
     have_checked_locale = False
 
@@ -53,13 +53,10 @@ class FacebookScraper:
             self.session.cookies.set("noscript", "0")
 
     def set_proxy(self, proxy):
-        self.requests_kwargs.update({
-            'proxies': {
-                'http': proxy,
-                'https': proxy
-            }
-        })
-        ip = self.get("http://ipinfo.io", headers={"Accept": "application/json"}).json()
+        self.requests_kwargs.update({'proxies': {'http': proxy, 'https': proxy}})
+        ip = self.get(
+            "http://lumtest.com/myip.json", headers={"Accept": "application/json"}
+        ).json()
         logger.debug(f"Proxy details: {ip}")
 
     def get_posts(self, account: str, **kwargs) -> Iterator[Post]:
@@ -80,31 +77,53 @@ class FacebookScraper:
                 url = url.replace(FB_W3_BASE_URL, FB_MOBILE_BASE_URL)
             if not url.startswith(FB_MOBILE_BASE_URL):
                 url = utils.urljoin(FB_MOBILE_BASE_URL, url)
-            post = {
-                "original_request_url": post_url,
-                "post_url": url
-            }
+            post = {"original_request_url": post_url, "post_url": url}
             logger.debug(f"Requesting page from: {url}")
             response = self.get(url)
             elem = response.html.find('[data-ft*="top_level_post_id"]', first=True)
             photo_post = False
             if response.html.find("div.msg", first=True):
                 photo_post = True
-                elem = response.html
+                elem = response.html.find("#root", first=True)
             if not elem:
                 logger.warning("No raw posts (<article> elements) were found in this page.")
             else:
-                comments_area = response.html.find('div[data-sigil="m-mentions-expand"]', first=True)
+                comments_area = response.html.find(
+                    'div.ufi', first=True
+                )
                 if comments_area:
                     # Makes likes/shares regexes work
-                    elem = utils.make_html_element(elem.html.replace("</footer>", comments_area.html + "</footer>"))
+                    elem = utils.make_html_element(
+                        elem.html.replace("</footer>", comments_area.html + "</footer>")
+                    )
 
                 if photo_post:
-                    post.update(extract_photo_post(elem, request_fn=self.get, options=options, full_post_html=response.html))
+                    post.update(
+                        extract_photo_post(
+                            elem,
+                            request_fn=self.get,
+                            options=options,
+                            full_post_html=response.html,
+                        )
+                    )
                 elif url.startswith(utils.urljoin(FB_MOBILE_BASE_URL, "/groups/")):
-                    post.update(extract_group_post(elem, request_fn=self.get, options=options, full_post_html=response.html))
+                    post.update(
+                        extract_group_post(
+                            elem,
+                            request_fn=self.get,
+                            options=options,
+                            full_post_html=response.html,
+                        )
+                    )
                 else:
-                    post.update(extract_post(elem, request_fn=self.get, options=options, full_post_html=response.html))
+                    post.update(
+                        extract_post(
+                            elem,
+                            request_fn=self.get,
+                            options=options,
+                            full_post_html=response.html,
+                        )
+                    )
                 if not post.get("post_url"):
                     post["post_url"] = url
                 if remove_source:
@@ -112,10 +131,36 @@ class FacebookScraper:
             yield post
 
     def get_profile(self, account, **kwargs) -> Profile:
+        result = {}
+
+        logger.debug(f"Requesting page from: {account}")
+        response = self.get(account)
+        match = re.search(r'entity_id:(\d+),', response.html.html)
+        if match:
+            result["id"] = match.group(1)
+        photo_links = response.html.find("a[href^='/photo.php']")
+        if photo_links:
+            cover_photo = photo_links[0]
+            result["cover_photo_text"] = cover_photo.attrs.get("title")
+            response = self.get(cover_photo.attrs.get("href"))
+            extractor = PostExtractor(response.html, kwargs, self.get)
+            result["cover_photo"] = extractor.extract_photo_link_HQ(response.html.html)
+
+            profile_photo = photo_links[1]
+            response = self.get(profile_photo.attrs.get("href"))
+            result["profile_picture"] = extractor.extract_photo_link_HQ(response.html.html)
+        else:
+            cover_photo = response.html.find(
+                "div[data-sigil='cover-photo']>i.img", first=True
+            ).attrs["style"]
+            match = re.search(r"url\('(.+)'\)", cover_photo)
+            if match:
+                result["cover_photo"] = utils.decode_css_url(match.groups()[0])
+            result["profile_picture"] = response.html.find("img.profpic", first=True).attrs["src"]
+
         about_url = utils.urljoin(FB_MOBILE_BASE_URL, f'/{account}/about/')
         logger.debug(f"Requesting page from: {about_url}")
         response = self.get(about_url)
-        result = {}
         # Profile name is in the title
         title = response.html.find("title", first=True).text
         if " | " in title:
@@ -129,7 +174,7 @@ class FacebookScraper:
         for card in about.find("div[data-sigil='profile-card']"):
             header = card.find("header", first=True).text
             if header.startswith("About"):
-                header = "About" # Truncate strings like "About Mark"
+                header = "About"  # Truncate strings like "About Mark"
             if header in ["Work, Education"]:
                 experience = []
                 for elem in card.find("div.experience"):
@@ -162,16 +207,14 @@ class FacebookScraper:
                     places.append(place)
                 result[header] = places
             else:
-                bits = card.text.split("\n")[1:] # Remove header
+                bits = card.text.split("\n")[1:]  # Remove header
                 if len(bits) >= 3 and header == "Relationship":
-                    result[header] = {
-                        "to": bits[0],
-                        "type": bits[1],
-                        "since": bits[2]
-                    }
+                    result[header] = {"to": bits[0], "type": bits[1], "since": bits[2]}
                 elif len(bits) == 1:
                     result[header] = bits[0]
-                elif header in ["Contact Info", "Basic info", "Other names"] and len(bits) % 2 == 0: # Divisible by two, assume pairs
+                elif (
+                    header in ["Contact Info", "Basic info", "Other names"] and len(bits) % 2 == 0
+                ):  # Divisible by two, assume pairs
                     pairs = {}
                     for i in range(0, len(bits), 2):
                         pairs[bits[i + 1]] = bits[i]
@@ -201,52 +244,99 @@ class FacebookScraper:
             for elem in elems:
                 name = elem.find("h3>a", first=True)
                 tagline = elem.find("div.notice.ellipsis", first=True).text
-                friends.append({
-                    "link": name.attrs.get("href"),
-                    "name": name.text,
-                    "tagline": tagline
-                })
+                profile_picture = elem.find("i.profpic", first=True).attrs.get("style")
+                match = re.search(r"url\('(.+)'\)", profile_picture)
+                if match:
+                    profile_picture = utils.decode_css_url(match.groups()[0])
+                user_id = json.loads(
+                    elem.find("a.touchable[data-store]", first=True).attrs["data-store"]
+                ).get("id")
+                friends.append(
+                    {
+                        "id": user_id,
+                        "link": name.attrs.get("href"),
+                        "name": name.text,
+                        "profile_picture": profile_picture,
+                        "tagline": tagline,
+                    }
+                )
             result["Friends"] = friends
         return result
 
     def get_page_info(self, page, **kwargs) -> Profile:
-        sample_post = next(self.get_posts(page, **kwargs))
-        resp = self.get(sample_post["post_id"])
-        elem = resp.html.find("script[type='application/ld+json']", first=True)
-        if not elem:
-            return None
-        meta = json.loads(elem.text)
-        result = meta["author"]
-        result["type"] = result.pop("@type")
-        desc = resp.html.find("meta[name='description']", first=True)
-        if desc:
-            match = re.search(r'([\d,]+)\s+likes', desc.attrs["content"])
-            if match:
-                result["likes"] = utils.parse_int(match.groups()[0])
-        for interaction in meta["interactionStatistic"]:
-            if interaction["interactionType"] == "http://schema.org/FollowAction":
-                result["followers"] = interaction["userInteractionCount"]
+        result = {}
+        for post in self.get_posts(page, **kwargs):
+            logger.debug(f"Fetching {post['post_id']}")
+            resp = self.get(post["post_id"])
+            elem = resp.html.find("script[type='application/ld+json']", first=True)
+            if not elem:
+                continue
+            meta = json.loads(elem.text)
+            if meta.get("creator"):
+                result = meta["creator"]
+                result["type"] = result.pop("@type")
+                desc = resp.html.find("meta[name='description']", first=True)
+                if desc:
+                    match = re.search(r'(\d[\d,.]+)', desc.attrs["content"])
+                    if match:
+                        result["likes"] = utils.parse_int(match.groups()[0])
+                try:
+                    for interaction in result.get("interactionStatistic", []):
+                        if interaction["interactionType"] == {
+                            "@type": "http://schema.org/FollowAction"
+                        }:
+                            result["followers"] = interaction["userInteractionCount"]
+                except TypeError as e:
+                    logger.error(e)
+                result.pop("interactionStatistic", None)
+                break
+
+        try:
+            about_url = f'/{page}/about/'
+            logger.debug(f"Requesting page from: {about_url}")
+            resp = self.get(about_url)
+            desc = resp.html.find("meta[name='description']", first=True)
+            if desc:
+                logger.debug(desc.attrs["content"])
+                match = re.search(r'(\d[\d,.]+)', desc.attrs["content"])
+                if match:
+                    result["likes"] = utils.parse_int(match.groups()[0])
+            result["about"] = resp.html.find('#pages_msite_body_contents', first=True).text
+        except Exception as e:
+            logger.error(e)
         return result
 
     def get_group_info(self, group, **kwargs) -> Profile:
         url = f'/groups/{group}'
         logger.debug(f"Requesting page from: {url}")
         resp = self.get(url).html
-        url = resp.find("a[href*='?view=info']", first=True).attrs["href"]
+        try:
+            url = resp.find("a[href*='?view=info']", first=True).attrs["href"]
+        except AttributeError:
+            raise exceptions.UnexpectedResponse("Unable to resolve view=info URL")
         logger.debug(f"Requesting page from: {url}")
         resp = self.get(url).html
         result = {}
         result["id"] = re.search(r'/groups/(\d+)', url).group(1)
-        result["name"] = resp.find("header h3", first=True).text
-        result["type"] = resp.find("header div", first=True).text
-        members = resp.find("div[data-testid='m_group_sections_members']", first=True)
-        result["members"] = utils.parse_int(members.text)
+        try:
+            result["name"] = resp.find("header h3", first=True).text
+            result["type"] = resp.find("header div", first=True).text
+            members = resp.find("div[data-testid='m_group_sections_members']", first=True)
+            result["members"] = utils.parse_int(members.text)
+        except AttributeError:
+            raise exceptions.UnexpectedResponse("Unable to get one of name, type, or members")
         url = members.find("a", first=True).attrs.get("href")
         logger.debug(f"Requesting page from: {url}")
         try:
             resp = self.get(url).html
             admins = resp.find("div:first-child>div.touchable a:not(.touchable)")
-            result["admins"] = [{"name": e.text, "link": e.attrs["href"].split("?")[0]} for e in admins]
+            result["admins"] = [
+                {
+                    "name": e.text,
+                    "link": utils.filter_query_params(e.attrs["href"], blacklist=["refid"]),
+                }
+                for e in admins
+            ]
             url = resp.find("a[href^='/browse/group/members']", first=True).attrs["href"]
             members = []
             while url:
@@ -259,7 +349,7 @@ class FacebookScraper:
                     url = more.group(1)
                 else:
                     url = None
-            result["members"] = [m for m in members if m not in result["admins"]]
+            result["other_members"] = [m for m in members if m not in result["admins"]]
         except exceptions.LoginRequired as e:
             pass
         return result
@@ -286,11 +376,22 @@ class FacebookScraper:
             response.html.html = response.html.html.replace('<!--', '').replace('-->', '')
             response.raise_for_status()
             self.check_locale(response)
-            if "noscript" not in response.html.html:
-                warnings.warn(f"Facebook served mbasic/noscript content unexpectedly on {response.url}")
+            if "cookie/consent-page" in response.url:
+                response = self.submit_form(response)
+            if (
+                response.url.startswith(FB_MOBILE_BASE_URL)
+                and "noscript" not in response.html.html
+                and self.session.cookies.get("noscript") != "1"
+            ):
+                warnings.warn(
+                    f"Facebook served mbasic/noscript content unexpectedly on {response.url}"
+                )
             title = response.html.find("title", first=True)
             not_found_titles = ["page not found", "content not found"]
-            temp_ban_titles = ["you can't use this feature at the moment", "you’re temporarily blocked"]
+            temp_ban_titles = [
+                "you can't use this feature at the moment",
+                "you’re temporarily blocked",
+            ]
             if title:
                 if title.text.lower() in not_found_titles:
                     raise exceptions.NotFound(title.text)
@@ -298,31 +399,44 @@ class FacebookScraper:
                     raise exceptions.TemporarilyBanned(title.text)
                 elif ">Your Account Has Been Disabled<" in response.html.html:
                     raise exceptions.AccountDisabled("Your Account Has Been Disabled")
-                elif ">We saw unusual activity on your account. This may mean that someone has used your account without your knowledge.<" in response.html.html:
+                elif (
+                    ">We saw unusual activity on your account. This may mean that someone has used your account without your knowledge.<"
+                    in response.html.html
+                ):
                     raise exceptions.AccountDisabled("Your Account Has Been Locked")
-                elif (title.text == "Log in to Facebook | Facebook" or
-                      response.url.startswith(utils.urljoin(FB_MOBILE_BASE_URL, "login")) or
-                      response.url.startswith(utils.urljoin(FB_W3_BASE_URL, "login")) or
-                      (", log in to Facebook." in response.text and not response.html.find("article[data-ft],div.async_like[data-ft],div.msg"))
-                      ):
-                    raise exceptions.LoginRequired("A login (cookies) is required to see this page")
+                elif (
+                    title.text == "Log in to Facebook | Facebook"
+                    or response.url.startswith(utils.urljoin(FB_MOBILE_BASE_URL, "login"))
+                    or response.url.startswith(utils.urljoin(FB_W3_BASE_URL, "login"))
+                    or (
+                        ", log in to Facebook." in response.text
+                        and not response.html.find(
+                            "article[data-ft],div.async_like[data-ft],div.msg"
+                        )
+                    )
+                ):
+                    raise exceptions.LoginRequired(
+                        "A login (cookies) is required to see this page"
+                    )
             return response
         except RequestException as ex:
             logger.exception("Exception while requesting URL: %s\nException: %r", url, ex)
             raise
 
-    def submit_form(self, response, extra_data = {}):
+    def submit_form(self, response, extra_data={}):
         action = response.html.find("form", first=True).attrs.get('action')
         url = utils.urljoin(self.base_url, action)
         elems = response.html.find("input[name][value]")
-        data = { elem.attrs['name']: elem.attrs['value'] for elem in elems }
+        data = {elem.attrs['name']: elem.attrs['value'] for elem in elems}
         data.update(extra_data)
         response = self.session.post(url, data=data)
         return response
 
     def login(self, email: str, password: str):
         response = self.get(self.base_url)
-        response = self.submit_form(response, {"email": email, "pass": password, "_fb_noscript": None})
+        response = self.submit_form(
+            response, {"email": email, "pass": password, "_fb_noscript": None}
+        )
 
         login_error = response.html.find('#login_error', first=True)
         if login_error:
@@ -362,7 +476,7 @@ class FacebookScraper:
         page_limit=DEFAULT_PAGE_LIMIT,
         options=None,
         remove_source=True,
-        **kwargs
+        **kwargs,
     ):
         counter = itertools.count(0) if page_limit is None else range(page_limit)
 
